% /******************* TO DO 5 *********************
%  * BlendImages:
% INPUT
%  ipv: a Matlab struct containing the images (in spherical coordinates,
%       with radial distortion removed), and global transformations
%       specifying for each image its position in the mosaic
%  blendWidth: width of the blending function
%  panoImgName: filename to which the final mosaic will be written
% 
% OUTPUT
%  none (but the final mosaic is saved to file)
% 
% This function blends all of the images into the final mosaic.  The images
% and their global translations are stored in a Matlab structure - you can
% create this structure using the writeipv function.  The writeipv function
% takes as input a cell array of the image names (of the prewarped images)
% and a cell array of transformation matrices between each pair of adjacent
% images.
% 
% First, you should figure out the extent of the mosaic, by computing the
% minimum and maximum x and y (really theta and phi) values.
% 
% Then, for each image, you should compute the sub-image of the mosaic at
% which the image should be written, and call AccumulateBlend to add the
% image here.  At the end, call NormalizeBlend.  What you're doing here is
% computing a weighted average of the pixel values at each location in the
% mosaic, where the weights are determined by distances to the horizontal
% centers of the original images.
% 
% In your panorama, the first image will appear at the beginning and end.
% As errors may have accumulated, the two copies of the image may not line
% up vertically.  As a final step, you should construct an affine
% transformation that shears the mosaic so that the two copies of the first
% image do line up vertically.  You should then use this transformation to
% warp the mosaic.
%  */

function BlendImages(ipv, blendWidth,panoImgName)
%  // Assume all the images are of the same shape (for now)

    
    img1 = ipv{1}.img;
    height       = size(img1,1);
    width        = size(img1,2);
    nBands       = size(img1,3);
   

%  // Compute the bounding box for the mosaic
    n = size(ipv,1);
    
    min_x = inf;
    min_y = inf;
    max_x = 0;
    max_y = 0;
    
    for i = 1: n
    
        pos = ipv{i}.position;
        
% 		// *** BEGIN TODO #1 ***
% 		// add some code here to update min_x, ..., max_y

        

% 		// *** END TODO #1 ***
    end

%     // Create a floating point accumulation image
    mShape=[(ceil(max_y) - floor(min_y)) (ceil(max_x) - floor(min_x))  nBands+1];
    accumulator=zeros(mShape);
    

%     // Add in all of the images
    for i = 1:n
    
%       // Compute the sub-image involved
        pos = ipv{i}.position;
        start_x = 0;
        start_y = 0;       

% 		// *** BEGIN TODO #2 ***
% 		// fill in with the correct values for start_x and start_y,
% 		// the relative position of i-th image in the mosaic
% 		// hint: we only require you to use pixel accuracy to start_x and start_y;


% 		// *** END TODO #2 ***

        img = ipv{i}.img;
        sub = accumulator(start_y+1:start_y+height ,start_x+1:start_x+width, :);

%       // Perform the accumulation
        sub=AccumulateBlend(img, sub, blendWidth);
        accumulator(start_y+1:start_y+height ,start_x+1:start_x+width, :)=sub;
        
       
    end

%     // Normalize the results
    compImage=zeros(mShape);
    compImage=NormalizeBlend(accumulator, compImage);
        
%     // Allocate the final image shape
    cShape=[mShape(1) (mShape(2) - width)   nBands];
    croppedImage=zeros(cShape);

%     // Compute the affine deformation
    A=eye(3);
    
% 	// *** BEGIN TODO #3 ***
%     // fill in the right entries in A to trim the left edge and
%     // to take out the vertical drift

   

% 	// *** END TODO #3 ***

%   // Warp and crop the composite
   [croppedImage]=WarpGlobal(compImage, croppedImage,A);
   croppedImage=uint8(croppedImage);
   imwrite(croppedImage(:,:,1:3),panoImgName,'png','BitDepth',8,'Alpha',reshape(croppedImage(:,:,4),[height width]));
 
    
