% /******************* TO DO *********************
%  * warpSphericalField:
% INPUT
%  srcImgName: filename from which the input image should be read
%  warpImgName: filename to which the output image should be written
%  f: camera focal length
%  k1,k2: camera radial distortion parameters
% 
% OUTPUT
%  none (but the warped image is saved to file)
% 
% This function transforms an image to spherical coordinates, also
% correcting for radial distortion.  The transformation is performed via an
% inverse warp, so your actual computation will involve transforming from
% spherical to image coordinates, and applying radial distortion.  The
% output image should be the same size as the input image.
%  */

function WarpSphericalField(srcImgName, warpImgName,f,k1, k2)
%     // Set up the pixel coordinate image
    srcImg0=imread(srcImgName);
    height=size(srcImg0,1);
    width=size(srcImg0,2);
    nBands=size(srcImg0,3);
    
    %append alpha channel
    if (nBands~=4)
        alphaChannel=255*ones(height,width,1);
        srcImg=zeros(height,width,4);
        srcImg(:,:,1:3)=srcImg0;
        srcImg(:,:,4)=alphaChannel;
        nBands=4;
    end
    
    
    uvImg=zeros(height,width,2); %// (u,v) coordinates
    

    for y =1:height       
        for x = 1:width
            
% 			// (x,y) is the spherical image coordinates. 
%           // (xf,yf) is the spherical coordinates, e.g., xf is the angle theta
% 			// and yf is the angle phi

            xf = ((x-1) - 0.5*width ) / f;
            yf = ((y-1) - 0.5*height) / f;

% 			// (xt,yt,zt) are intermediate coordinates to which you can
% 			// apply the spherical correction and radial distortion
            xt=0;
            yt=0;
            zt=0;
			p=zeros(3,1);
 
% 			// *** BEGIN TODO ***
% 			// add code to apply the spherical correction, i.e.,
% 			// compute the Euclidean coordinates
% 			// and project the point to the z=1 plane at
% 			// (xt/zt,yt/zt,1), then distort with radial distortion
% 			// coefficients k1 and k2
            
            
% 			// *** END TODO ***

%           // Convert back to regular pixel coordinates and store
            xn = 0.5*width  + xt*f;
            yn = 0.5*height + yt*f;
            
            uvImg(y,x,1)=xn;
            uvImg(y,x,2)=yn;
            
            
        end
    end
   
    %output
    warpImg=zeros(height,width,nBands);
    for n = 1:nBands
       warpImg(:,:,n) = interp2( srcImg(:,:,n), uvImg(:,:, 1), uvImg(:,:,2) );
    end
    warpImg(isnan(warpImg) ) = 0; 
    warpImg=uint8(warpImg);
    imwrite(warpImg(:,:,1:3),warpImgName,'png','BitDepth',8,'Alpha',reshape(warpImg(:,:,4),[height width]));


